/***************************************************************************
 *   Copyright (C) 2004 by Jens Luetkens                                   *
 *   j.luetkens@limitland.de                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <kconfig.h>

#include "Comix.h"
#include "ComixClient.h"

using namespace COMIX;

int ComixHandler::m_borderSize		= 5;
int ComixHandler::m_penWidth		= 2;

//
// Constructor
//
ComixHandler::ComixHandler()
{
	reset(1);
}

//
// Destructor
//
ComixHandler::~ComixHandler()
{
	m_initialized = false;
}

//
// createDecoration( KDecorationBcomix *b )
//
KDecoration* ComixHandler::createDecoration( KDecorationBridge *b )
{
    return new ComixClient(b, this);
}

/*
//
// supports( Ability ability )
//
bool ComixHandler::supports( Ability ability )
{
    switch( ability )
    {
        case AbilityAnnounceButtons:
        case AbilityButtonMenu:
        case AbilityButtonOnAllDesktops:
        case AbilityButtonSpacer:
        case AbilityButtonHelp:
        case AbilityButtonMinimize:
        case AbilityButtonMaximize:
        case AbilityButtonClose:
        case AbilityButtonAboveOthers:
        case AbilityButtonBelowOthers:
        case AbilityButtonShade:
            return true;
        default:
            return false;
    };

}
*/

//
// reset(unsigned long changed)
//
bool ComixHandler::reset(unsigned long changed)
{

    // read in the configuration
    m_initialized = false;

	// border sizese
    switch(KDecoration::options()->preferredBorderSize( this )) {
        case BorderTiny:
            m_borderSize = 3;
			m_penWidth = 2;
            break;
        case BorderLarge:
            m_borderSize = 8;
			m_penWidth = 3;
            break;
        case BorderVeryLarge:
            m_borderSize = 12;
			m_penWidth = 4;
            break;
        case BorderHuge:
            m_borderSize = 18;
			m_penWidth = 5;
            break;
        case BorderVeryHuge:
            m_borderSize = 26;
			m_penWidth = 6;
            break;
        case BorderOversized:
            m_borderSize = 40;
			m_penWidth = 7;
            break;
        case BorderNormal:
        default:
            m_borderSize = 5;
			m_penWidth = 2;
    }

    // reset all clients
    resetDecorations( changed );

    m_initialized = true;

    return true;
}

//
// borderSizes()
//
QValueList< ComixHandler::BorderSize >
ComixHandler::borderSizes() const
{
    // the list must be sorted
    return QValueList< BorderSize >() << BorderTiny << BorderNormal <<
	BorderLarge << BorderVeryLarge <<  BorderHuge <<
	BorderVeryHuge << BorderOversized;

}

//
// Plugin Stuff
//
extern "C"
{
    KDecorationFactory* create_factory()
    {
        return new ComixHandler();
    }
}

