/***************************************************************************
 *   Copyright (C) 2004 by Jens Luetkens                                   *
 *   j.luetkens@limitland.de                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <math.h>

#include <qlayout.h>
#include <qlabel.h>
#include <qpainter.h>
#include <qimage.h>
#include <qtooltip.h>
#include <qsettings.h>
#include <qimage.h>

#include <kconfig.h>
#include <kstandarddirs.h>
#include <kapplication.h>
#include <klocale.h>
#include <kpixmap.h>
#include <kpixmapeffect.h>

#include <kdecoration.h>
#include <kdecorationfactory.h>

#include "ComixClient.h"
#include "ComixButton.h"


using namespace COMIX;

// Default button layout
const char default_left[]  = "M";
const char default_right[] = "HIAX";

//
// Constructor
//
ComixClient::ComixClient( KDecorationBridge *b, KDecorationFactory *f )
	    : KDecoration( b, f ),
		m_windowBackgroundColor(0,0,0),
		m_windowContourColor(0,0,0),
		mainLayout_(0),
		titleSpacer_(0),
		menuIcon(0),
		borderTL(0), borderTR(0), borderBL(0), borderBR(0),
		titleWidth(0),
		topHeight(0),
		fontHeight(0),
		textMargin(0),
		boxHeight(0),
		titleBarColor(0,0,0),
		titleBlendColor(0,0,0),
		boxRadius(0),
		countButtons(0),
		countSpacer(0),
		borderSize(0),
		penWidth(0),
		hasMenu(false)
{
}

//
// Destructor
//
ComixClient::~ComixClient()
{

    deletePixmaps();

    for (int n=0; n<ButtonCount; n++) {
        if (m_buttons[n]) delete m_buttons[n];
    }

}

//
// init(void)
//
void ComixClient::init( void )
{
    createMainWidget(WResizeNoErase | WRepaintNoErase);
    widget()->installEventFilter(this);

    widget()->setBackgroundMode(NoBackground);
	
	readConfig();

    createLayout();

    createPixmaps();

}

//
// readConfig()
//
bool ComixClient::readConfig()
{

    // create a config object
    KConfig config("kwincomixrc");
    config.setGroup("General");

    QString value = config.readEntry("TitleAlignment", "AlignLeft");
    if (value == "AlignLeft")         m_titleAlign = Qt::AlignLeft;
    else if (value == "AlignHCenter") m_titleAlign = Qt::AlignHCenter;
    else if (value == "AlignRight")   m_titleAlign = Qt::AlignRight;

	// contour color
	QSettings settings;
	m_contrast = settings.readNumEntry("/Qt/KDE/contrast", 4);

	// no difference between active and inactive background/contour, use active here
	// (there is no difference in windows content as well)
	m_windowBackgroundColor = options()->color(KDecoration::ColorHandle, true);
	m_windowContourColor = options()->color(KDecoration::ColorFrame, true);

	someGeometry();

    return true;
}

//
// createPixmaps()
//
void ComixClient::createPixmaps()
{

	borderCircle = new ComixCircle(5,2);	// fixed

	//
	// if hasMenu
	//
	menuIcon = icon().pixmap(QIconSet::Small, QIconSet::Normal);
	menuIcon.convertFromImage( menuIcon.convertToImage().smoothScale(fontHeight-1, fontHeight-1));


	// tiles
	QImage btlImg = QImage( 5, 5, 32 );
	QImage btrImg = QImage( 5, 5, 32 );
	QImage bblImg = QImage( 5, 5, 32 );
	QImage bbrImg = QImage( 5, 5, 32 );

	btlImg.fill(0);
	btrImg.fill(0);
	bblImg.fill(0);
	bbrImg.fill(0);

	btlImg.setAlphaBuffer(true);
	btrImg.setAlphaBuffer(true);
	bblImg.setAlphaBuffer(true);
	bbrImg.setAlphaBuffer(true);

	PixelMap *thePixel;
	QRgb rgb = m_windowContourColor.rgb();
	unsigned int rgba;
	int reversealpha = 0;


	if( borderCircle ) {

		thePixel = borderCircle->fillMap;
		while( thePixel ) {
			// solid draw
			rgba = qRgba(qRed(rgb),qGreen(rgb),qBlue(rgb),255);
			// top left
			btlImg.setPixel( thePixel->x, thePixel->y, rgba );
			btlImg.setPixel( thePixel->y, thePixel->x, rgba );
			// top right
			btrImg.setPixel( 4 - thePixel->x, thePixel->y, rgba );
			btrImg.setPixel( 4 - thePixel->y, thePixel->x, rgba );
			// bottom left
			bblImg.setPixel( thePixel->x, 4 - thePixel->y, rgba );
			bblImg.setPixel( thePixel->y, 4 - thePixel->x, rgba );
			// bottom right
			bbrImg.setPixel( 4 - thePixel->x, 4 - thePixel->y, rgba );
			bbrImg.setPixel( 4 - thePixel->y, 4 - thePixel->x, rgba );

			thePixel = thePixel->next;
		}

		thePixel = borderCircle->blackMap;
		while( thePixel ) {

			rgba = qRgba(qRed(rgb),qGreen(rgb),qBlue(rgb),thePixel->alpha);

			// top left
			btlImg.setPixel( thePixel->x, thePixel->y, rgba );
			btlImg.setPixel( thePixel->y, thePixel->x, rgba );
			// top right
			btrImg.setPixel( 4 - thePixel->x, thePixel->y, rgba );
			btrImg.setPixel( 4 - thePixel->y, thePixel->x, rgba );
			// bottom left
			bblImg.setPixel( thePixel->x, 4 - thePixel->y, rgba );
			bblImg.setPixel( thePixel->y, 4 - thePixel->x, rgba );
			// bottom right
			bbrImg.setPixel( 4 - thePixel->x, 4 - thePixel->y, rgba );
			bbrImg.setPixel( 4 - thePixel->y, 4 - thePixel->x, rgba );

			thePixel = thePixel->next;
		}

		thePixel = borderCircle->whiteMap;
		while( thePixel ) {

			reversealpha = 255 - thePixel->alpha;
			rgba = qRgba(qRed(rgb),qGreen(rgb),qBlue(rgb),reversealpha);

			// top left
			btlImg.setPixel( thePixel->x, thePixel->y, rgba );
			btlImg.setPixel( thePixel->y, thePixel->x, rgba );
			// top right
			btrImg.setPixel( 4 - thePixel->x, thePixel->y, rgba );
			btrImg.setPixel( 4 - thePixel->y, thePixel->x, rgba );
			// bottom left
			bblImg.setPixel( thePixel->x, 4 - thePixel->y, rgba );
			bblImg.setPixel( thePixel->y, 4 - thePixel->x, rgba );
			// bottom right
			bbrImg.setPixel( 4 - thePixel->x, 4 - thePixel->y, rgba );
			bbrImg.setPixel( 4 - thePixel->y, 4 - thePixel->x, rgba );

			thePixel = thePixel->next;
		}

		borderTL = QPixmap(btlImg);
		borderTR = QPixmap(btrImg);
		borderBL = QPixmap(bblImg);
		borderBR = QPixmap(bbrImg);

	}

}

//
// deletePixmaps()
//
void ComixClient::deletePixmaps()
{

	delete borderCircle;
	
}

//
// eventFilter( QObject *obj, QEvent *e )
//
bool ComixClient::eventFilter( QObject *obj, QEvent *e )
{
    if (obj != widget()) return false;

    switch (e->type()) {
      case QEvent::MouseButtonDblClick: {
          mouseDoubleClickEvent(static_cast<QMouseEvent *>(e));
          return true;
      }
      case QEvent::MouseButtonPress: {
          processMousePressEvent(static_cast<QMouseEvent *>(e));
          return true;
      }
      case QEvent::Paint: {
          paintEvent(static_cast<QPaintEvent *>(e));
          return true;
      }
      case QEvent::Resize: {
          resizeEvent(static_cast<QResizeEvent *>(e));
          return true;
      }
      case QEvent::Show: {
          showEvent(static_cast<QShowEvent *>(e));
          return true;
      }
      default: {
          return false;
      }
    }

    return false;
}

//
// paintEvent(QPaintEvent *)
//
void ComixClient::paintEvent(QPaintEvent *)
{
	QPainter p(widget());
	QFontMetrics fm(options()->font(isActive(), false));
	
	titleBarColor = options()->color(KDecoration::ColorTitleBar, isActive());
	titleBlendColor = options()->color(KDecoration::ColorTitleBlend, isActive());

	QRect titleRect(titleSpacer_->geometry());
	QRect windowRect(0,0,width(),height());

	if ( isSetShade() ) {
		windowRect.setHeight(boxHeight + borderSize + borderSize);
	}	
	setMask(windowRect);

	frameWindowRect(windowRect, &p);

	// top background
	insetRect(&windowRect, borderSize);
	if( !isSetShade() ) windowRect.setHeight(boxHeight+borderSize);
	p.fillRect(windowRect, m_windowBackgroundColor);


	// title geometry
	titleRect.setHeight(boxHeight);
	titleRect.setWidth(titleRect.width()-borderSize);
	int maxTitleWidth = titleRect.width();

	int drawWidth = fm.width(caption()) + textMargin + textMargin + penWidth + penWidth + penWidth;
	if ( hasMenu ) drawWidth += textMargin + menuIcon.width();
	titleWidth = QMIN(drawWidth, titleRect.width());
	titleRect.setWidth(titleWidth);

	int maxMoveBy = maxTitleWidth-titleRect.width();
	int wantMoveBy = (windowRect.width() - titleRect.width() - windowRect.left() - titleRect.left()) / 2;

	switch( m_titleAlign ) {

        case Qt::AlignHCenter:
			titleRect.moveBy(QMIN(maxMoveBy,wantMoveBy),0);
			break;

        case Qt::AlignRight:
			titleRect.moveBy(maxMoveBy,0);
			break;

		default:
			break;

	}



	// title Rect
	QRect frameTitleRect = titleRect;
	insetRect(&titleRect, 2);

	decorateComixRect(titleRect, &p, titleBarColor);

	frameComixRect(frameTitleRect, &p);

	insetRect(&titleRect, textMargin);


	// draw icon
	if ( hasMenu ) {
	    p.drawPixmap(titleRect.left(), titleRect.top(), menuIcon);
		titleRect.setLeft(titleRect.left()+menuIcon.width()+textMargin);
	}
	titleRect.setTop(titleRect.top()-(textMargin/2));
	titleRect.setHeight(titleRect.height()+(textMargin/2));


	//draw text
	// isTool ?
	p.setFont(options()->font(isActive(), false));
	p.setPen(options()->color(KDecoration::ColorFont, isActive()));
	p.drawText(titleRect, AlignLeft, caption());
	
}

//
// decorateComixRect( QRect rect, QPainter * p, QColor backgroundColor )
//
void ComixClient::decorateComixRect( QRect rect, QPainter * p, QColor backgroundColor )
{

	p->fillRect(rect, backgroundColor);

	int grad = 0;
	grad = (3 * m_contrast) + 100;
	p->setPen(backgroundColor.light(grad));
	p->drawLine(rect.left(),rect.top(),rect.right(),rect.top());
	p->setPen(backgroundColor.dark(grad));
	p->drawLine(rect.left(),rect.bottom(),rect.right(),rect.bottom());

	grad = (2 * m_contrast) + 100;
	p->setPen(backgroundColor.light(grad));
	p->drawLine(rect.left(),rect.top()+1,rect.right(),rect.top()+1);
	p->setPen(backgroundColor.dark(grad));
	p->drawLine(rect.left(),rect.bottom()-1,rect.right(),rect.bottom()-1);

	grad = (m_contrast) + 100;
	p->setPen(backgroundColor.light(grad));
	p->drawLine(rect.left(),rect.top()+2,rect.right(),rect.top()+2);
	p->setPen(backgroundColor.dark(grad));
	p->drawLine(rect.left(),rect.bottom()-2,rect.right(),rect.bottom()-2);

}

//
// frameComixRect( QRect rect, QPainter * p 
//
void ComixClient::frameComixRect( QRect rect, QPainter * p )
{

	p->setPen(m_windowContourColor);
	
	p->drawLine(rect.left()+5,rect.top(),rect.right()-5,rect.top());
	p->drawLine(rect.left()+5,rect.top()+1,rect.right()-5,rect.top()+1);
	p->drawLine(rect.left()+5,rect.bottom(),rect.right()-5,rect.bottom());
	p->drawLine(rect.left()+5,rect.bottom()-1,rect.right()-5,rect.bottom()-1);
	p->drawLine(rect.left(),rect.top()+5,rect.left(),rect.bottom()-5);
	p->drawLine(rect.left()+1,rect.top()+5,rect.left()+1,rect.bottom()-5);
	p->drawLine(rect.right(),rect.top()+5,rect.right(),rect.bottom()-5);
	p->drawLine(rect.right()-1,rect.top()+5,rect.right()-1,rect.bottom()-5);

	p->drawPixmap(rect.left(), rect.top(), borderTL);
	p->drawPixmap(rect.right() - 5 + 1, rect.top(), borderTR);
	p->drawPixmap(rect.left(), rect.bottom() - 5 + 1, borderBL);
	p->drawPixmap(rect.right() - 5 + 1, rect.bottom() - 5 + 1, borderBR);

}

//
// frameWindowRect( QRect rect, QPainter * p 
//
void ComixClient::frameWindowRect( QRect rect, QPainter * p )
{

	p->setPen(m_windowContourColor);

	int i = 0;
	while( i < (penWidth) ) {
		p->drawRect(rect);
		insetRect(&rect,1);
		i++;
	}
	
	p->setPen(m_windowBackgroundColor);

	i = 0;
	while( i < (borderSize - penWidth) ) {
		p->drawRect(rect);
		insetRect(&rect,1);
		i++;
	}
	

}

//
// insetRect( QRect * rect, int inset )
//
void ComixClient::insetRect( QRect * rect, int inset )
{

	rect->setWidth(rect->width()-inset-inset);
	rect->setHeight(rect->height()-inset-inset);
	rect->moveBy(inset,inset);

}

//
// mouseDoubleClickEvent(QMouseEvent * e)
//
void ComixClient::mouseDoubleClickEvent(QMouseEvent * e)
{

    if (titleSpacer_->geometry().contains(e->pos()))
		titlebarDblClickOperation();

}

void ComixClient::captionChange(void)
{
    widget()->repaint(titleSpacer_->geometry(), false);
}

//
// desktopChange()
//
void ComixClient::desktopChange()
{
    bool s = isOnAllDesktops();
    if (m_buttons[ButtonOnAllDesktops])
    {
        m_buttons[ButtonOnAllDesktops]->update();
        m_buttons[ButtonOnAllDesktops]->setOnAllDesktops(s);
        QToolTip::add(m_buttons[ButtonOnAllDesktops], s ? i18n("Not On All Desktops") : i18n("On All Desktops"));
    }
}


//
// maximizeChange()
//
void ComixClient::maximizeChange()
{
    widget()->repaint( false );
}

//
// activeChange()
//
void ComixClient::activeChange()
{
	widget()->repaint(false);
	repaintButtons();
}

//
// resize (const QSize &size)
//
void ComixClient::resize (const QSize &size)
{
    widget()->resize(size);
}

//
// borders(int &l, int &r, int &t, int &b) const
//
void ComixClient::borders(int &l, int &r, int &t, int &b) const
{

	t = topHeight;
	b = l = r = borderSize;
	
}

//
// createButtons()
//
void ComixClient::createButtons(QBoxLayout *layout, const QString& buttons)
{

	for (unsigned int i = 0; i < buttons.length(); ++i)
	{
		switch (buttons[i].latin1())
		{
		case 'H': // Help
			if ((!m_buttons[ButtonHelp]) && providesContextHelp()){
				m_buttons[ButtonHelp] = new ComixButton(this, "help", i18n("Help"), ButtonHelp, false);
				connect(m_buttons[ButtonHelp], SIGNAL(clicked()), this, SLOT(showContextHelp()));
				layout->addWidget(m_buttons[ButtonHelp], 0,  Qt::AlignTop);
				layout->setStretchFactor(m_buttons[ButtonHelp], 0);
				m_buttons[ButtonHelp]->setFixedSize(boxHeight, boxHeight);
				countButtons++;
            }
			break;
		case 'M': // Menu
			hasMenu = true;
			break;
		case 'S': // On All Desktops (Sticky)
			m_buttons[ButtonOnAllDesktops] = new ComixButton(this, "onalldesktops", i18n("On All Desktops"), ButtonOnAllDesktops, true);
			connect(m_buttons[ButtonOnAllDesktops], SIGNAL(clicked()), this, SLOT(toggleOnAllDesktops()));
			layout->addWidget(m_buttons[ButtonOnAllDesktops], 0,  Qt::AlignTop);
			layout->setStretchFactor(m_buttons[ButtonOnAllDesktops], 0);
			m_buttons[ButtonOnAllDesktops]->setFixedSize(boxHeight, boxHeight);
			countButtons++;
			break;
		case 'I': // Minimize
			if (isMinimizable()){
				m_buttons[ButtonIconify] = new ComixButton(this, "iconify", i18n("Minimize"), ButtonIconify, false);
				connect(m_buttons[ButtonIconify], SIGNAL(clicked()), this, SLOT(minimize()));
				layout->addWidget(m_buttons[ButtonIconify], 0,  Qt::AlignTop);
				layout->setStretchFactor(m_buttons[ButtonIconify], 0);
				m_buttons[ButtonIconify]->setFixedSize(boxHeight, boxHeight);
				countButtons++;
			}
			break;
		case 'A': // Maximize
			if (isMaximizable()){
				m_buttons[ButtonMaximize] = new ComixButton(this, "maximize", i18n("Maximize"), ButtonMaximize, true);
				connect(m_buttons[ButtonMaximize], SIGNAL(clicked()), this, SLOT(maxButtonPressed()));
				layout->addWidget(m_buttons[ButtonMaximize], 0,  Qt::AlignTop);
				layout->setStretchFactor(m_buttons[ButtonMaximize], 0);
				m_buttons[ButtonMaximize]->setFixedSize(boxHeight, boxHeight);
				countButtons++;
			}
			break;
		case 'X': // Close
			if (isCloseable()){
				m_buttons[ButtonClose] = new ComixButton(this, "close", i18n("Close"), ButtonClose, false);
				connect(m_buttons[ButtonClose], SIGNAL(clicked()), this, SLOT(closeWindow()));
				layout->addWidget(m_buttons[ButtonClose], 0, Qt::AlignTop);
				layout->setStretchFactor(m_buttons[ButtonClose], 0);
				m_buttons[ButtonClose]->setFixedSize(boxHeight,boxHeight);
				countButtons++;
			}
			break;
		case 'L': // Shade
			m_buttons[ButtonShade] = new ComixButton(this, "shade", i18n("Shade"), ButtonShade, true);
			connect(m_buttons[ButtonShade], SIGNAL(clicked()), this, SLOT(slotShade()));
			layout->addWidget(m_buttons[ButtonShade], 0, Qt::AlignTop);
			layout->setStretchFactor(m_buttons[ButtonShade], 0);
			m_buttons[ButtonShade]->setFixedSize(boxHeight, boxHeight);
			countButtons++;
			break;
		case 'F': // Above all others
			m_buttons[ButtonAbove] = new ComixButton(this, "above", i18n("Keep Above Others"), ButtonAbove, true);
			connect(m_buttons[ButtonAbove], SIGNAL(clicked()), this, SLOT(slotKeepAbove()));
			layout->addWidget(m_buttons[ButtonAbove], 0, Qt::AlignTop);
			layout->setStretchFactor(m_buttons[ButtonAbove], 0);
			m_buttons[ButtonAbove]->setFixedSize(boxHeight, boxHeight);
			countButtons++;
			break;
		case 'B': // Below all others
			m_buttons[ButtonBelow] = new ComixButton(this, "below", i18n("Keep Below Others"), ButtonBelow, true);
			connect(m_buttons[ButtonBelow], SIGNAL(clicked()), this, SLOT(slotKeepBelow()));
			layout->addWidget(m_buttons[ButtonBelow], 0, Qt::AlignTop);
			layout->setStretchFactor(m_buttons[ButtonBelow], 0);
			m_buttons[ButtonBelow]->setFixedSize(boxHeight, boxHeight);
			countButtons++;
			break;
		case '_': // Spacer item
			layout->addSpacing(borderSize);
			countSpacer++;
			break;
		}

	}

}

//
// someGeometry()
//
void ComixClient::someGeometry() 
{
	
	QFontMetrics fm(options()->font(isActive(), false));
	
	textMargin = QMAX(3,fm.height() / 8);
	fontHeight = QMAX(8, fm.height());
	boxHeight = fontHeight + textMargin + textMargin;

	// must be a multiple of 2
	if (! boxHeight%2==0 )
		boxHeight += 1;
	
	boxRadius = boxHeight / 2;
	
	borderSize = ComixHandler::borderSize();
	penWidth = ComixHandler::penWidth();

	topHeight = boxHeight + borderSize + borderSize;

}

//
// createLayout()
//
void ComixClient::createLayout()
{

    // basic layout:
    //  ________________________________________________________________
    // |  _______ ____________________________________________ ________ |
    // | | btns | |         titleSpacer                      | | btns | |
    // | |______| |__________________________________________| |___ __| |
    // |   ___________________________________________________________  |
    // | |                      contentsFake                          | |
    // | |                                                            | |
    // | |                                                            | |
    // | |____________________________________________________________| |
    // |________________________________________________________________|
    //

    delete mainLayout_;

    mainLayout_ = new QVBoxLayout(widget(), borderSize, (borderSize - penWidth));

    titleSpacer_      = new QSpacerItem(32, boxHeight,  QSizePolicy::Expanding, QSizePolicy::Fixed);

    // title
    QHBoxLayout *titleLayout = new QHBoxLayout(mainLayout_, borderSize, 0);

    // sizeof(...) is calculated at compile time
    memset(m_buttons, 0, sizeof(ComixButton *) * ButtonCount);

    createButtons(titleLayout, options()->customButtonPositions() ? options()->titleButtonsLeft() : QString(default_left));
	
    titleLayout->addItem(titleSpacer_);
	
    createButtons(titleLayout, options()->customButtonPositions() ? options()->titleButtonsRight() : QString(default_right));
    //Mid
    QHBoxLayout * midLayout   = new QHBoxLayout(mainLayout_, 0, 0);
    if( isPreview())
        midLayout->addWidget(new QLabel( i18n( "<center><b>comix preview</b></center>" ), widget()) );
    else
        midLayout->addItem( new QSpacerItem( 0, 0 ));

	// Make sure that topLayout doesn't stretch - midLayout should take
	// all spare space.
	mainLayout_->setStretchFactor(titleLayout, 0);
	mainLayout_->setStretchFactor(midLayout, 1);

}

//
// repaintButtons(void)
//
void ComixClient::repaintButtons(void)
{
	//force all buttons to repaint
    for (int n=0; n<ButtonCount; n++)
        if (m_buttons[n]) m_buttons[n]->repaint(false);

}

//
// maxButtonPressed()
//
void ComixClient::maxButtonPressed()
{
    if (m_buttons[ButtonMaximize]) {
        switch (m_buttons[ButtonMaximize]->lastMousePress())
        {
          case MidButton:
              maximize(maximizeMode() ^ MaximizeVertical );
              maximizeChange();
              break;
          case RightButton:
              maximize(maximizeMode() ^ MaximizeHorizontal );
              maximizeChange();
              break;
          default:
              maximize(maximizeMode() == MaximizeFull ? MaximizeRestore : MaximizeFull );
              maximizeChange();
        }

        m_buttons[ButtonMaximize]->setMaximized( maximizeMode()!=MaximizeRestore );
        QToolTip::add(m_buttons[ButtonMaximize], (maximizeMode()!=MaximizeRestore) ? i18n("Restore") : i18n("Maximize"));
    }
}

//
// mousePosition(const QPoint &point) const
//
KDecoration::Position ComixClient::mousePosition(const QPoint &point) const
{
	return KDecoration::mousePosition(point);
}

//
// minimumSize() const
//
QSize ComixClient::minimumSize() const
{
    return QSize (100, 50);
}

//
// iconChange()
//
void ComixClient::iconChange()
{
    if (hasMenu)
    {
	    widget()->repaint(titleSpacer_->geometry(), false);
    }
}

//
// shadeChange ()
//
void ComixClient::shadeChange ()
{
    if ( m_buttons[ButtonShade] ) {
        bool shaded = isSetShade();
        m_buttons[ButtonShade]->setOn( shaded );
//        m_buttons[ButtonShade]->setTipText( shaded ?
//                i18n("Unshade")
//                : i18n("Shade"));
        QToolTip::add(m_buttons[ButtonShade], shaded ? i18n("Unshade") : i18n("Shade"));
    }
}

//
// slotShade()
//
void ComixClient::slotShade()
{
	
	setShade( !isSetShade() );

}

//
// slotKeepAbove()
//
void ComixClient::slotKeepAbove()
{
    bool above = !keepAbove();
    setKeepAbove( above );
    if (m_buttons[ButtonAbove])
    {
        m_buttons[ButtonAbove]->setOn(above);
        m_buttons[ButtonAbove]->setAbove(above);
        QToolTip::add(m_buttons[ButtonAbove], above ? i18n("Do Not Keep Above Others") : i18n("Keep Above Others"));
    }

    if (m_buttons[ButtonBelow] && m_buttons[ButtonBelow]->isOn())
    {
        m_buttons[ButtonBelow]->setOn(false);
        m_buttons[ButtonBelow]->setBelow(false);
        QToolTip::add(m_buttons[ButtonBelow], i18n("Keep Below Others"));
    }
}

//
// slotKeepBelow()
//
void ComixClient::slotKeepBelow()
{
    bool below = !keepBelow();
    setKeepBelow( below );
    if (m_buttons[ButtonBelow])
    {
        m_buttons[ButtonBelow]->setOn(below);
        m_buttons[ButtonBelow]->setBelow(below);
        QToolTip::add(m_buttons[ButtonBelow], below ? i18n("Do Not Keep Below Others") : i18n("Keep Below Others"));
    }

    if (m_buttons[ButtonAbove] && m_buttons[ButtonAbove]->isOn())
    {
        m_buttons[ButtonAbove]->setOn(false);
        m_buttons[ButtonAbove]->setAbove(false);
        QToolTip::add(m_buttons[ButtonAbove], i18n("Keep Above Others"));
    }
}

//
// resizeEvent(QResizeEvent *)
//
void ComixClient::resizeEvent(QResizeEvent *)
{
    widget()->repaint();
}

//
// showEvent(QShowEvent *)
//
void ComixClient::showEvent(QShowEvent *)
{
    widget()->repaint();
}

//
// isTool() const
//
bool ComixClient::isTool() const
{
    const int SUPPORTED_WINDOW_TYPES_MASK =
        NET::NormalMask | NET::DesktopMask | NET::DockMask | NET::ToolbarMask |
        NET::MenuMask | NET::DialogMask | NET::OverrideMask | NET::UtilityMask |
        NET::SplashMask;
        NET::WindowType type = windowType (SUPPORTED_WINDOW_TYPES_MASK);
    return type == NET::Toolbar || type == NET::Utility || type == NET::Menu;
}

